// Copyright 2002 Mad Doc Software, LLC.  All rights reserved.
// $Header: //StarTrek/RTM/PC/src/lib/core/LAllocator.h#1 $
// $Modtime: 12/28/04 4:03p $

#ifndef ALLOCATOR_H
#define ALLOCATOR_H



class LMemoryPool;	



class LStandardMem {
public:
	static void *Malloc(int n)
	{
		return new char[n];
	}
	static void Free(void *p)
	{
		delete[] p;
	}

	static void *Realloc(void *p, int nOld, int nNew)
	{
		void *pNew = new char[nNew];
		memcpy(pNew, p, nOld < nNew ? nOld: nNew);
		delete[] p;
		return pNew;
	}
};


#define MALLOC(sz)					LStandardMem::Malloc(sz)
#define REALLOC(ptr, nOld, nNew)	LStandardMem::Realloc(ptr, nOld, nNew)
#define FREE(ptr)					LStandardMem::Free(ptr)
#define SAFE_FREE(ptr) do if (ptr) {FREE(ptr); (ptr) = 0;} while (0)


void CheckHeap();

//	-------------------------------------------------------------------
//		LTempMem
//	-------------------------------------------------------------------

class LTempMem
{
public:
	static void *Malloc(int n);
	static void Free(void *p);
	static void *Realloc(void *p, int nOld, int nNew);

	static bool AreAnyInUse();	
};



class IAllocator
{
public:
	virtual void *Malloc(int size) = 0;
	virtual void *Realloc(void *p, int nOld, int nNew) = 0;
	virtual void Free(void *p) = 0;
};



class LDefaultAllocator : public IAllocator
{
public:
	virtual void *Malloc(int n)
	{
		return new char[n];
	}

	virtual void Free(void *p)
	{
		delete[] p;
	}

	virtual void *Realloc(void *p, int nOld, int nNew)
	{
		void *pNew = new char[nNew];
		memcpy(pNew, p, nOld < nNew ? nOld: nNew);
		delete[] p;
		return pNew;
	}
};



class LPoolAllocator : public IAllocator
{
public:
	
	LPoolAllocator(LMemoryPool *pool)
	{
		m_pool = pool;
	}

	

	virtual void *Malloc(int n);
	virtual void Free(void *p);
	virtual void *Realloc(void *p, int nOld, int nNew);

private:
	LMemoryPool *m_pool;		
};



class LStaticAllocator : public IAllocator
{
public:
	LStaticAllocator(char *buff, int buffSize)
	{
		m_buff = buff;
		m_curr = buff;
		m_size = buffSize;
	}

	
	virtual void *Malloc(int n);
	virtual void Free(void *p);
	virtual void *Realloc(void *p, int nOld, int nNew);

private:
	char *m_buff;		// ptr to start of buffer
	char *m_curr;		
	int m_size;			
};



class LTempAllocator : public IAllocator
{
public:
	
	LTempAllocator();
	
	~LTempAllocator();

	virtual void *Malloc(int n)
	{
		EnterCriticalSection( &m_CriticalSection );
		void* pRetVal = LTempMem::Malloc(n);
		LeaveCriticalSection( &m_CriticalSection );
		return pRetVal;
	}

	virtual void Free(void *p)
	{
		EnterCriticalSection( &m_CriticalSection );
		LTempMem::Free(p);
		LeaveCriticalSection( &m_CriticalSection );
	}

	virtual void *Realloc(void *p, int nOld, int nNew)
	{
		EnterCriticalSection( &m_CriticalSection );
		void* pRetVal = LTempMem::Realloc(p, nOld, nNew);
		LeaveCriticalSection( &m_CriticalSection );
		return pRetVal;
	}
	
	static LTempAllocator& TheTempAllocator();

private:
	
	CRITICAL_SECTION m_CriticalSection;
};



extern LDefaultAllocator g_defaultAllocator;


#endif 
