

#ifndef LDYNARRAY_H
#define LDYNARRAY_H

#include "LDebugMemory.h"



#include "LAllocator.h"
#include "LError.h"
#include "LSizedArray.h"



template <class DATATYPE> 
class LDynArrayBase : public LSizedArray<DATATYPE>
{
public:
	
	LDynArrayBase()
	{
	}

	
	LDynArrayBase(int capacity, IAllocator *allocator = NULL) : LSizedArray<DATATYPE>(0, NULL)
	{
		m_capacity = capacity;
		m_allocator = allocator;
		if (capacity)
		{
			if (m_allocator)
				m_pData = (DATATYPE *)m_allocator->Malloc(m_capacity * sizeof(DATATYPE));
			else
				m_pData = (DATATYPE *) LStandardMem::Malloc(m_capacity * sizeof(DATATYPE));
		}
		else
			m_pData = NULL;
	}

	LDynArrayBase(int capacity) : LSizedArray<DATATYPE>(0, NULL)
	{
		m_capacity = capacity;
		m_allocator = NULL;
		if (capacity)
		{
			m_pData = (DATATYPE *) LStandardMem::Malloc(m_capacity * sizeof(DATATYPE));
		}
		else
			m_pData = NULL;
	}

	
	~LDynArrayBase() 
	{
		Reset();
	}

	
	void SetAllocator(IAllocator *allocator)
	{
		ASSERT(m_pData == NULL);
		m_allocator = allocator;
	}
	
	
	int Append(DATATYPE elem)
	{
        int elemIndex = Grow();
		m_pData[elemIndex] = elem;	
        return elemIndex;
	}

	
	int AppendByReference(const DATATYPE &elem)
	{
        int elemIndex = Grow();
		m_pData[elemIndex] = elem;	
        return elemIndex;
	}

	
    int Append(const LSizedArray<DATATYPE> &src)
    {
		if (src.GetSize() > 0)
			return Append(src.GetPtr(), src.GetSize());
		return m_size;	
    }

	
    int Append(const DATATYPE *src, int size)
    {
        int startIndex = Grow(size);
        DATATYPE *dest = &m_pData[startIndex];
		
		if( src )
		{
			for (int i = 0; i < size; i++)
				dest[i] = src[i];
		}
        return startIndex;
    }

	
	DATATYPE *Append()
	{
		int index = Grow(1);
		return &m_pData[index];
	}

	
	DATATYPE Pop()
	{
		ASSERT(m_size > 0);
		return m_pData[--m_size];
	}

	// Pop: remove last element from array and return it into referenced value
	void Pop(DATATYPE &value)
	{
		ASSERT(m_size > 0);
		value = m_pData[--m_size];
	}

	// Remove: remove an element, does not preserve ordering (replaces element
	// with last element in array).
	void RemoveNoOrder(int i)
	{
		ASSERT((0<=i) && (i<m_size));
		if (i < (m_size - 1))
			m_pData[i] = m_pData[m_size - 1];
		m_size--;
	}

	
	void Remove(int i)
	{
		ASSERT((0<=i) && (i<m_size));
		for (int j = i; j < m_size-1 ; ++j)
			m_pData[j] = m_pData[j+1];
		m_size--;
	}

	
	void FindAndRemove(const DATATYPE &findMe)
	{
		int index = Find(findMe);
		if (index >= 0)
			Remove(index);
	}
	
	
	void RemoveRange(int iStart, int iEnd)
	{
		ASSERT(0<=iStart && iStart<=iEnd && iEnd<m_size);
		int iNumRemoved = iEnd - iStart + 1;
		for(int i = iStart; i < m_size - iNumRemoved; i++)
			m_pData[i] = m_pData[i + iNumRemoved];
		m_size -= iNumRemoved;
	}

	
	void Insert(const DATATYPE &elem, int index)
	{
		ASSERT((0<=index) && (index<=m_size));		
		Grow();								
		for (int i = m_size - 1; i > index; --i)
			m_pData[i] = m_pData[i-1];		
		m_pData[index] = elem;				
	}

	
	void Insert(const DATATYPE *src, int size, int index)
	{
		ASSERT((0<=index) && (index<=m_size));
		Grow(size);							
		for (int i = m_size - 1; i-size >= index; --i)
			m_pData[i] = m_pData[i-size];	
		for (int j = index; j < (index+size); j++)
			m_pData[j] = *src++;			// insert new ones starting at index
	}
	
	
	void InsertMultiple(const DATATYPE &in_elem, int in_iHowMany, int in_iIndex)
	{
		ASSERT((0<=in_iIndex) && (in_iIndex<=m_size));
		Grow(in_iHowMany);					
		for (int i = m_size - 1; i-in_iHowMany >= in_iIndex; --i)
			m_pData[i] = m_pData[i-in_iHowMany];	
		for (int j = in_iIndex; j < (in_iIndex+in_iHowMany); j++)
			m_pData[j] = in_elem;			
	}

	
	void InsertInOrder(const DATATYPE &elem)
	{
		for (int i = 0; i < m_size; i++)	
		{
			if (elem < m_pData[i])			
				break;
		}
		Insert(elem, i);					// and insert before it
	}
	
	
	void InsertInOrderBinarySearch(const DATATYPE &elem)
	{
		int iInsertIndex = -1;
		int iFindIndex = BinarySearch(elem, &iInsertIndex);

		
		if(iFindIndex != -1)
			iInsertIndex = iFindIndex;

		
		Insert(elem, iInsertIndex);
	}
	
	
	template<class PREDICATE>
	void InsertInOrder(const DATATYPE &elem, const PREDICATE &pred)
	{
		for(int i = 0; i < m_size; i++)			
		{
			if(pred.Less(elem, m_pData[i]))		
				break;
		}
		Insert(elem, i);						
	}

	
	template <class COMPARECLASS>
	void InsertInOrderBinarySearch(const DATATYPE &elem, const COMPARECLASS &comparer)
	{
		int iInsertIndex = -1;
		int iFindIndex = BinarySearchCustomCompare(elem, comparer, &iInsertIndex);
		
		
		if(iFindIndex != -1)
			iInsertIndex = iFindIndex;
		
		
		Insert(elem, iInsertIndex);
	}

	
	int Grow(int n = 1) 
	{
		ASSERT(n >= 0);
		int elemIndex = m_size;
		SetSize(m_size + n);
		return elemIndex;
	}

	
	void SetCapacity(int newCapacity)
	{
		ASSERT(m_size <= newCapacity);
		if (m_capacity == 0 && newCapacity>0)
		{
			if (m_allocator)
				m_pData = (DATATYPE *)m_allocator->Malloc(newCapacity * sizeof(DATATYPE));
			else
				m_pData = (DATATYPE *) LStandardMem::Malloc(newCapacity * sizeof(DATATYPE));
		}
		else if (newCapacity > 0)
		{
			if (m_allocator)
				m_pData = (DATATYPE *)m_allocator->Realloc(m_pData, m_size * sizeof(DATATYPE), newCapacity * sizeof(DATATYPE));
			else
			{
				DISABLE_UNINITIALIZED_CHECK();
				m_pData = (DATATYPE *)LStandardMem::Realloc(m_pData, m_size * sizeof(DATATYPE), newCapacity * sizeof(DATATYPE));
			}
		}
		else if (m_pData)
		{
			if (m_allocator)
				m_allocator->Free(m_pData);
			else
			{
				DISABLE_UNINITIALIZED_CHECK();
				LStandardMem::Free(m_pData);
			}
			m_pData = NULL;
		}
		m_capacity = newCapacity;
	}

	
	void SetSize(int n)
	{
		if (n > m_capacity)
			SetCapacity(n + (n/4) + 4 + 16/sizeof(DATATYPE));
		m_size = n;
	}

	
	void Clear()
	{
		m_size = 0;
	}

	// Reset: set size and capacity to zero
	void Reset()
	{
		m_size = 0;
		m_capacity = 0;
		if (m_pData)
		{
			if (m_allocator)
				m_allocator->Free(m_pData);
			else
			{
				
				DISABLE_UNINITIALIZED_CHECK();
				LStandardMem::Free(m_pData);
			}
			m_pData = 0;
		}
	}

	
	int GetCapacity(void) const 
	{ 
		return m_capacity; 
	}

	
    void Copy(const DATATYPE *src, int size)
    {
        SetSize(0);
        Append(src, size);
    }

	// Copy: overwrite array with new elements.  If source array is empty, then
	// just reset this array.
    void Copy(const LSizedArray<DATATYPE> &src)
    {
		if (src.GetSize() > 0)
			Copy(src.GetPtr(), src.GetSize());
		else
			Reset();
    }

	// Fill: fill the array with n copies of the same value
    void Fill(int size, const DATATYPE& value)
    {
        SetSize(size);
        for (int i = 0; i < size; i++)
            m_pData[i] = value;
    }

    
    void SetExactSize(int n)
    {
        m_size = 0;			
		if (m_capacity != n)
			SetCapacity(n);		
		m_size = n;			
    }

	
	void DeleteAllAndReset()
	{
		
		for (int i = 0; i < m_size ; ++i)
			delete m_pData[i];
		
		Reset();
	}
	
	
	void DeleteAllAndClear()
	{
		
		for (int i = 0; i < m_size ; ++i)
			delete m_pData[i];
		
		Clear();
	}
	
	
	void ReleaseAllAndReset()
	{
		
		for (int i=0; i < m_size; i++)
		{
			SafeRelease(m_pData[i]);
		}
		Reset();
	}
	
	
	void ReleaseAllAndClear()
	{
		
		for (int i=0; i < m_size; i++)
		{
			SafeRelease(m_pData[i]);
		}
		Clear();
	}

	
	DATATYPE* DetachData()
	{
		DATATYPE* pData = m_pData;
		m_size = 0;
		m_capacity = 0;
		m_pData = NULL;
		return pData;
	}

protected:
	int m_capacity;				
	IAllocator *m_allocator;	

private:
	
	LDynArrayBase(LDynArrayBase<DATATYPE> const &in_rhs);
	LDynArrayBase &operator =(LDynArrayBase<DATATYPE> const &in_rhs);
};



template <class DATATYPE>
class LDynArray : public LDynArrayBase<DATATYPE>
{
public:
	
	LDynArray(int capacity, IAllocator *allocator) : LDynArrayBase<DATATYPE>(capacity, allocator) 
	{
	}

	
	LDynArray(int capacity = 0) : LDynArrayBase<DATATYPE>(capacity, NULL) 
	{
	}

private:
	
	LDynArray(LDynArray<DATATYPE> const &in_rhs);
	LDynArray &operator =(LDynArray<DATATYPE> const &in_rhs);
};



template <class DATATYPE>
class LTempArray : public LDynArray<DATATYPE>
{
public:
	LTempArray(int capacity = 0) : LDynArray<DATATYPE>(capacity, &LTempAllocator::TheTempAllocator()) 
	{
	}

private:
	
	LTempArray(LTempArray<DATATYPE> const &in_rhs);
	LTempArray &operator =(LTempArray<DATATYPE> const &in_rhs);
};



template <class DATATYPE>
class LStaticArray : public LDynArray<DATATYPE>
{
public:
	
	LStaticArray(char *buff, int buffSize) : LDynArray<DATATYPE>(0, &m_privateAllocator),
		m_privateAllocator(buff, buffSize)
	{
		SetCapacity(buffSize / sizeof(DATATYPE));
	}
private:
	LStaticAllocator m_privateAllocator;

	
	LStaticArray(LStaticArray<DATATYPE> const &in_rhs);
	LStaticArray &operator =(LStaticArray<DATATYPE> const &in_rhs);
	LStaticArray();
};


#include "LDebugMemory.h"


template<class DATATYPE> class LGlobalDynArray : public LDynArrayBase<DATATYPE>, public LDebugMemory::CleanupCallback
{
public:
	virtual void Cleanup()
	{
		Reset();
	}
};

#endif // __DYNARRAY_H__
