
#ifndef MISCMATH_H
#define MISCMATH_H
#pragma once

#include <math.h>        

inline bool IsEqual( float a, float b, float epsilon = EPSILON )  { return Abs(a-b) < epsilon; }
inline bool IsZero(  float r, float epsilon = EPSILON )           { return Abs(r)   < epsilon; }





template <class T>
inline float Lerp( float a, float b, float s )     { return a + (b-a)*s; }

template <class T>
inline float Lerp( const T&a, const T&b, float s )     { return a + (b-a)*s; }



inline float AngularLerp(float in_fAngle1, float in_fAngle2, float in_fFrac)
{
	while( in_fAngle2<in_fAngle1 )
	{
		in_fAngle2 += TWO_PI;
	}
	return Lerp(in_fAngle1, in_fAngle2, in_fFrac);
}


template <class T>
inline T LerpMap(float x, float x1, float x2, const T&y1, const T&y2) 
{
	ASSERT(x2 > x1);
	if (x <= x1) 
		return y1;
	else if (x >= x2)
		return y2;

	return Lerp( y1, y2 ,(x - x1)/(x2 - x1));
} 



inline float InterpolateQuadSlowStart(float in_fValue1, float in_fValue2, float in_fPercent1to2)
{
	
	return in_fValue1 + in_fPercent1to2 * in_fPercent1to2 * (in_fValue2 - in_fValue1);
}


inline float InterpolateQuadFastStart(float in_fValue1, float in_fValue2, float in_fPercent1to2)
{
	
	return in_fValue1 + (-1 * in_fPercent1to2 * in_fPercent1to2 + 2 * in_fPercent1to2) * (in_fValue2 - in_fValue1);
}



inline float RoundToNearest( float in_fValue, float in_fIncrement )
{
	ASSERT(in_fIncrement > 0.0f);

	
	return (float)Round(in_fValue / in_fIncrement) * in_fIncrement;
}



template <class T>
inline T Min( const T& a, const T& b )
{ return (a <= b)  ?  a  :  b; }

template <class T>
inline T Max( const T& a, const T& b )
{ return (a >= b)  ?  a  :  b; }


template <class T>
inline bool MinEq( T* pA, const T& b )
{
	if (b < *pA) {
		*pA = b;
		return true;
	}
	return false;
}


template <class T>
inline bool MaxEq( T* pA, const T& b )
{ 
	if (b > *pA) {
		*pA = b; 
		return true;
	}
	return false;
}



template <class T> 
inline T Clamp( const T& val, const T& min, const T& max )
{ 
	return (val <= min) ? min : ((val >= max) ? max : val); 
}

//---------------------------------------------------------------------------
// ClampEq - clamp val in place to range [min, max], returns true if assigned
//---------------------------------------------------------------------------
template <class T> 
inline bool ClampEq( T& io_val, const T& min, const T& max )
{ 
	if (io_val < min) 
	{
		io_val = min;
		return true;
	}
	else if (io_val > max) 
	{
		io_val = max;
		return true;
	}
	return false;
}


template <class T> 
inline void Swap( T& a, T& b )
{ 
	T tmp = a;  
	a = b;  
	b = tmp; 
}


template <class T>
inline T Sign(const T& val) 
{
	return (val < 0) ? -1 : ((val > 0) ? 1 : 0) ;
}




inline int Modulus(int x, int n)
{
	ASSERT(n > 0);
	return (x % n < 0) ? n + x % n : x % n;
}



inline bool IsPowerOfTwo( uint32 x )
{
	return x && ((x & (x-1)) == 0);
} 



inline uint32 PopulateOnes( uint32 x )
{
	x |= (x>>1);
	x |= (x>>2);
	x |= (x>>4);
	x |= (x>>8);
	x |= (x>>16);
	return x;
}
 

inline uint32 NearestPowerTwo( uint32 x )
{
	
	if ((x & (x-1)) == 0)
		return x;

	
	return PopulateOnes(x)+1;
} 



inline uint32 PopulationCount(uint32 x)
{
	
	x -= ((x >> 1) & 0x55555555);
	
	x = (((x >> 2) & 0x33333333) + (x & 0x33333333));
	// add sets of 4 bits
	x = (((x >> 4) + x) & 0x0f0f0f0f);
	
	x += (x >> 8);
	
	x += (x >> 16);
	
	return (x & 0x0000003f);
}


inline uint32 Log2(uint32 x)
{
	x = PopulateOnes(x);
	return PopulationCount(x) - 1;
}

#endif MISCMATH_H


 