

#ifndef LOG_H
#define LOG_H

class LStr;		


#define LOGF_ENABLED	0x0001		
#define LOGF_OUTDBGSTR	0x0002		
#define LOGF_FLUSH		0x0004		
#define LOGF_TIMESTAMP	0x0008		


enum eLogSeverity
{
	kLogSeverity_Spew,		
	kLogSeverity_Log,		
	kLogSeverity_Warn,		
	kLogSeverity_Error,		
	kLogSeverity_Fatal,		

	kNumLogSeverities
};

eLogSeverity Log_StrToSeverity(const char* in_str);

#ifdef LOGGING_ON



#define LOGSOURCE_DECLARE(name) LogSource gLS_##name(#name)



#define LOGSOURCE_REFERENCE(name) extern LogSource gLS_##name

extern const char *g_errorFile;		
extern int g_errorLine;			


#define ERRSETFL(file,line) do {g_errorFile = file; g_errorLine = line;} while (0)





#define LOGWARN(logsrc, level, x) LOGWARN_INDIRECT(__FILE__, __LINE__, logsrc, level, x)


#define LOG(logsrc, x) LOGWARN(logsrc, kLogSeverity_Log, x)


#define LOGWARN_INDIRECT(file, line, logsrc, level, x) \
do {	\
	if ((level >= gLS_##logsrc.m_LoggingSeverity && gLS_##logsrc.m_Enabled) || (level >= gLS_##logsrc.m_WarningSeverity && g_warningsEnabled)) \
{if (g_warningsEnabled) ERRSETFL(file, line);gLS_##logsrc.m_CurrentLoggingSeverity = level;gLS_##logsrc.Write x;} \
} while (0)


#define LOGRAW(logsrc, x) do {if (gLS_##logsrc.m_Enabled) gLS_##logsrc.WriteRaw(x);} while (0)



#define LOGFLUSH(logsrc) gLS_##logsrc.Flush();



#define LOG_CONFIG(filename) LogSource::Config(filename)
#define LOG_CONFIG_STATEMENT(parser) LogSource::ConfigStatement(parser)


#define OUTPUT_DEBUG_STRING(x) LogSource::WriteToOutputDebugString x



#define LOGSRC(logsrc) gLS_##logsrc



class LHandParser;
class LStr;

class LogSource 
{
public:
	
	LogSource(const char *in_Name);

	
	~LogSource();

	
	void Write(const char *in_FormatString, ...) const;
	
	
	void WriteArgList(const char* in_FormatString, va_list in_argList) const;
	
	
	void WriteRaw(const char *in_pRawString) const;

	// Write to all log sources
	static void WriteAll(const char *in_FormatString, ...);

	
	static void WriteToOutputDebugString(const char *in_FormatString, ...);

	
	void SetEnabled() 
	{
		m_Enabled = 1; m_Flags |= LOGF_ENABLED;
	}

	void SetDisabled() 
	{
		m_Enabled = 0; m_Flags &= ~LOGF_ENABLED;
	}

	void SetFlags(uint32 in_Flags)
	{
		m_Flags = in_Flags; m_Enabled = (in_Flags & LOGF_ENABLED) ? 1 : 0;
	}

	void AddFlags(uint32 in_Flags)
	{
		m_Flags |= in_Flags; m_Enabled = (in_Flags & LOGF_ENABLED) ? 1 : 0;
	}

	void RemoveFlags(uint32 in_Flags)
	{
		m_Flags &= ~in_Flags; m_Enabled = (in_Flags & LOGF_ENABLED) ? 1 : 0;
	}

	
	bool IsEnabled() const 
	{
		return m_Enabled;
	}

	bool IsFlush() const 
	{
		return (m_Flags & LOGF_FLUSH) ? 1 : 0;
	}

	bool IsTimestamp() const 
	{
		return (m_Flags & LOGF_TIMESTAMP) ? 1 : 0;
	}

	bool IsOutDbgStr() const 
	{
		return (m_Flags & LOGF_OUTDBGSTR) ? 1 : 0;
	}

	uint32 GetLogFileMask() const 
	{
		return m_LogFileMask;
	}

	
	void SetWarnSeverity(eLogSeverity in_severity)
	{
		m_WarningSeverity = in_severity;
	}
	void SetLoggingSeverity(eLogSeverity in_severity)
	{
		m_LoggingSeverity = in_severity;
	}


	
	int GetNumLogFiles() const;

	// Flush the log source
	void Flush() const;

	// Add or remove logfiles
	bool AddLogFile(const char *in_pFilename);
	bool RemoveLogFile(const char *in_pFilename);
	void RemoveAllLogFiles();

	
	static bool RestartLogFile(const char *in_pFilename);

	
	void GetFileNameList(LStr &fileList) const;

	
	static void FlushAllLogFiles();

	
	static LogSource* Find(const char *in_Name);

	
	static LogSource *GetHead() 
	{
		return s_HeadLogSource;
	}

	LogSource *GetNext() 
	{
		return m_Next;
	}

	// Is this log source equal to, or a child of, the supplied name?
	bool IsFamily(const char *in_FamilyName);
	bool IsFamily(LogSource &in_LogSource) 
	{
		return IsFamily(in_LogSource.m_Name);
	}

	
	static void Config(const char *in_CfgFilename);
	static bool ConfigStatement(LHandParser *in_Parser);

private:
	
	void RemoveFromLinkedList();

	
	static LogSource *s_HeadLogSource;

	LogSource *	m_Next;		
	const char *m_Name;		
	uint32		m_LogFileMask;	
	uint32		m_Flags;			

	
public:
	
	eLogSeverity m_LoggingSeverity;

	
	eLogSeverity m_CurrentLoggingSeverity;

	
	eLogSeverity m_WarningSeverity;

	bool		m_Enabled;			
};


#else	

#define LOGSOURCE_DECLARE(name)
#define LOGSOURCE_REFERENCE(name)
#define LOGWARN(logsrc, level, x)
#define LOGWARN_INDIRECT(file, line, logsrc, level, x) 
#define LOG(logsrc, x)
#define LOGRAW(logsrc, x)
#define LOGFLUSH(logsrc)
#define LOG_CONFIG(filename)
#define LOG_CONFIG_STATEMENT(parser)
#define OUTPUT_DEBUG_STRING(x)

#define ERRSETFL(file,line)

#define LOGSRC(x) g_dummyLog


class LogSource
{
public:
	
	bool IsEnabled() const
	{
		return false;
	}

	
	void Write(const char *in_FormatString, ...) const
	{
	}
};

extern LogSource g_dummyLog;

#endif	

#endif  
