


#ifndef VEC2F_H
#define VEC2F_H

#include "LMathConsts.h" 
#include "LFastFloat.h" 

class NiPoint2;

class Vec2f
{
public:
	float x;
	float y;

	Vec2f() {}	

	Vec2f(float inX,float inY) : x(inX), y(inY) {}

	operator NiPoint2&() { return *(NiPoint2*)this; }	
	operator const NiPoint2&() const { return *(NiPoint2*)this; }	// NiPoint2 cast operator.

	NiPoint2& GetNiPoint2() { return *(NiPoint2*)this; }	
	const NiPoint2& GetNiPoint2() const { return *(NiPoint2*)this; }	

	
	void Set(float newX, float newY) {
		x = newX; y = newY;
	}

	
	void SetZero() {
		x = y = 0;
	}

	
	bool operator== (const Vec2f& inVector) const { 
		return (inVector.x == x && inVector.y == y); 
	}
	
	bool operator!= (const Vec2f& inVector) const { 
		return (inVector.x != x || inVector.y != y); 
	}

	
	bool IsEqual(const Vec2f& in_vector, float in_epsilon = EPSILON) const;

	Vec2f operator+ (const Vec2f& inVector) const { 
		return Vec2f(x+inVector.x, y+inVector.y); 
	}

	Vec2f& operator+= (const Vec2f& inVector) { 
		x += inVector.x;
		y += inVector.y;
		return *this; 
	}

	Vec2f operator- (const Vec2f& inVector) const { 
		return Vec2f(x-inVector.x, y-inVector.y); 
	}

	Vec2f operator- () const {
		return Vec2f(-x,-y);
	}

	Vec2f& operator-= (const Vec2f& inVector) { 
		x -= inVector.x;
		y -= inVector.y;
		return *this;
	}

	Vec2f operator* (const float scale) const { 
		return Vec2f(x*scale, y*scale); 
	}

	Vec2f& operator*= (const float scale) {
		x *= scale; 
		y *= scale; 
		return *this;
	}

	Vec2f operator/ (const float scale) const { 
		return Vec2f(x/scale, y/scale); 
	}

	Vec2f& operator/= (const float scale) { 
		x /= scale; 
		y /= scale; 
		return *this; 
	}

	float& operator[] (int dir) { 
		ASSERT(dir >= 0 && dir < 2);
		return *(&x+dir); 
	}

	const float& operator[] (int dir) const	{ 
		ASSERT(dir >= 0 && dir < 2);
		return *(&x+dir); 
	}

	float Length() const { return sqrt(x*x+y*y); }

	float LengthSqr() const { return (x*x+y*y); }

	float Distance(const Vec2f& inVector) const {
		const float dx = x-inVector.x;
		const float dy = y-inVector.y;
		return sqrt(dx*dx + dy*dy);
	}

	float DistanceSqr(const Vec2f& inVector) const {
		const float dx = x-inVector.x;
		const float dy = y-inVector.y;
		return dx*dx + dy*dy;
	}

	
	float LinearLengthApproximation() const 
	{
		float fx = (float)FABS(x);
		float fy = (float)FABS(y);
		return (fx < fy)?(fy + .3f*fx):(fx + .3f*fy);
	}

	float LinearDistanceApproximation(const Vec2f& vPos) const 
	{
		float fx = (float)FABS(x - vPos.x);
		float fy = (float)FABS(y - vPos.y);
		return (fx < fy)?(fy + .3f*fx):(fx + .3f*fy);
	}

	
	float QuadraticLengthApproximation() const 
	{
		float fx = (float)FABS(x);
		float fy = (float)FABS(y);

		
		const float kCoeff = .457f;

		if (fx > fy)
		{
			return fx + kCoeff*fy*fy/fx;
		}
		
		if (fy==0)
			return 0;

		return fy + kCoeff*fx*fx/fy;
	}

	float Dot(const Vec2f& inVector) const
		{ return x*inVector.x + y*inVector.y; }

	float Cross(const Vec2f& inVector) const
		{ return x*inVector.y - y*inVector.x;  }

	
	float Normalize()
	{
		float len = Length();
		if (len > 0)
		{
			float fReciprocal = 1.0/len;
			x *= fReciprocal;  y *= fReciprocal;
		}
		return len;
	}

	
	float GetAngleZ() const;
	
	
	Vec2f RotateZ(double fAngle) const;

	
	inline Vec2f RotateZ90() const
	{
		return Vec2f(-y,x);
	}
	
	
	inline Vec2f RotateZ180() const
	{
		return Vec2f(-x,-y);
	}
	
	
	inline Vec2f RotateZ270() const
	{
		return Vec2f(y,-x);
	}
};


class Vec2fConst
{
public:
	
	static const Vec2f ZERO;	
	static const Vec2f UNIT_X;	
	static const Vec2f UNIT_Y;	
	static const Vec2f UNIT_ALL; //1,1
};
	
	

// Scalar multiply the other way, silly c++
inline Vec2f operator* (float fScalar,const Vec2f &rhs) {
	return rhs * fScalar;
}





#endif 
